/*
 * Copyright 2009 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.lemnik.eodsql.impl;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.sql.SQLException;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import net.lemnik.eodsql.impl.BaseQueryImpl.Callable;
import net.lemnik.eodsql.impl.BaseQueryImpl.ConnectionSource;
import net.lemnik.eodsql.impl.BaseQueryImpl.MethodImpl;

/**
 * A class for translating checked into unchecked exceptions.
 * 
 * @author Bernd Rinn
 */
public class ExceptionTranslator
{
    private final static boolean haveSpring = checkHaveSpring();

    private static boolean checkHaveSpring()
    {
        try
        {
            Class.forName("org.springframework.jdbc.support.SQLExceptionTranslator");
            return true;
        } catch (ClassNotFoundException ex)
        {
            return false;
        }
    }

    private final SpringSQLExceptionTranslatorWrapper translatorOrNull;

    ExceptionTranslator(ConnectionSource source)
    {
        if (haveSpring)
        {
            translatorOrNull = new SpringSQLExceptionTranslatorWrapper(source);
        } else
        {
            translatorOrNull = null;
        }
    }

    /**
     * Throw the given exception, wrapping it in some sort of unchecked exception.
     */
    RuntimeException translateException(final String task, final String sql, final Exception ex)
    {
        if (translatorOrNull != null && ex instanceof SQLException)
        {
            return translatorOrNull.translate(task, sql, (SQLException) ex);
        } else
        {
            return new RuntimeException(ex);
        }
    }

    /**
     * Throw the given exception, wrapping it in some sort of unchecked exception.
     */
    RuntimeException translateException(final Method method, final Callable callable,
            final Exception ex)
    {
        if (translatorOrNull != null && ex instanceof SQLException)
        {
            final String methodName = method.getName();
            final String sql;
            if (callable instanceof MethodImpl)
            {
                final AbstractMethodImplementation<Annotation> m =
                        ((MethodImpl) callable).getMethodImpl();
                sql = m.query.toString();
            } else
            {
                sql = "?";
            }
            return translatorOrNull.translate(methodName, sql, (SQLException) ex);
        } else
        {
            return new RuntimeException(ex);
        }
    }

    /**
     * Returns an exception when a unique result is expected from the database, but the database
     * returns multiple rows.
     */
    public static RuntimeException uniqueResultExpected()
    {
        final String msg = "A unique result was expected but the database returned multiple rows.";
        if (haveSpring)
        {
            return new IncorrectResultSizeDataAccessException(msg, 1);
        } else
        {
            return new RuntimeException(msg);
        }
    }
}
